@@ -52,6 +52,7 @@ KateFileBrowser::KateFileBrowser(KTextEditor::MainWindow *mainWindow, QWidget *p
     m_toolbar->setMovable(false);
     m_toolbar->setToolButtonStyle(Qt::ToolButtonIconOnly);
     m_toolbar->setContextMenuPolicy(Qt::NoContextMenu);
+    m_toolbar->layout()->setContentsMargins(0, 0, 0, 0);
 
     // ensure reasonable icons sizes, like e.g. the quick-open and co. icons
     // the normal toolbar sizes are TOO large, e.g. for scaled stuff even more!
@@ -114,6 +115,8 @@ KateFileBrowser::KateFileBrowser(KTextEditor::MainWindow *mainWindow, QWidget *p
     connect(m_mainWindow, &KTextEditor::MainWindow::viewChanged, this, &KateFileBrowser::autoSyncFolder);
 
     connect(m_dirOperator, &KDirOperator::contextMenuAboutToShow, this, &KateFileBrowser::contextMenuAboutToShow);
+
+    autoSyncFolder();
 }
 
 KateFileBrowser::~KateFileBrowser()
@@ -159,7 +162,7 @@ void KateFileBrowser::readSessionConfig(const KConfigGroup &cg)
 
     m_urlNavigator->setLocationUrl(cg.readEntry("location", QUrl::fromLocalFile(QDir::homePath())));
     setDir(cg.readEntry("location", QUrl::fromLocalFile(QDir::homePath())));
-    m_autoSyncFolder->setChecked(cg.readEntry("auto sync folder", false));
+    m_autoSyncFolder->setChecked(cg.readEntry("auto sync folder", true));
     m_filter->setHistoryItems(cg.readEntry("filter history", QStringList()), true);
 }
 
@@ -379,6 +382,7 @@ void KateFileBrowser::setupActions()
     m_autoSyncFolder = new QAction(this);
     m_autoSyncFolder->setCheckable(true);
     m_autoSyncFolder->setText(i18n("Automatically synchronize with current document"));
+    m_autoSyncFolder->setChecked(true);
     m_autoSyncFolder->setIcon(QIcon::fromTheme(QStringLiteral("system-switch-user")));
     connect(m_autoSyncFolder, &QAction::triggered, this, &KateFileBrowser::autoSyncFolder);
     optionsMenu->addAction(m_autoSyncFolder);
@@ -399,5 +403,3 @@ void KateFileBrowser::setupActions()
     }
 }
 // END Protected
-
-// kate: space-indent on; indent-width 2; replace-tabs on;
