@@ -27,6 +27,7 @@ public:
     ~KateFileTree() override;
 
     void setModel(QAbstractItemModel *model) override;
+    void setShowCloseButton(bool show);
 
 public Q_SLOTS:
     void slotDocumentClose();
@@ -72,6 +73,8 @@ private Q_SLOTS:
 
     void slotRenameFile();
 
+    void onRowsMoved(const QModelIndex &parent, int start, int end, const QModelIndex &destination, int row);
+
 private:
     QAction *setupOption(QActionGroup *group, const QIcon &, const QString &, const QString &, const char *slot, bool checked = false);
 
@@ -98,6 +101,8 @@ private:
 
     QPersistentModelIndex m_previouslySelected;
     QPersistentModelIndex m_indexContextMenu;
+
+    bool m_hasCloseButton;
 };
 
 #endif // KATE_FILETREE_H
