@@ -31,6 +31,11 @@
 #include <QPainter>
 #include <QVariant>
 
+static bool isUncomittedLine(const QByteArray &hash)
+{
+    return hash == "hash" || hash == "0000000000000000000000000000000000000000";
+}
+
 GitBlameInlineNoteProvider::GitBlameInlineNoteProvider(KateGitBlamePluginView *pluginView)
     : KTextEditor::InlineNoteProvider()
     , m_pluginView(pluginView)
@@ -53,7 +58,6 @@ QVector<int> GitBlameInlineNoteProvider::inlineNotes(int line) const
 
     QPointer<KTextEditor::Document> doc = m_pluginView->activeDocument();
     if (!doc) {
-        qDebug() << "no document";
         return QVector<int>();
     }
 
@@ -83,13 +87,13 @@ void GitBlameInlineNoteProvider::paintInlineNote(const KTextEditor::InlineNote &
     int lineNr = note.position().line();
     const CommitInfo &info = m_pluginView->blameInfo(lineNr);
 
+    bool isToday = info.authorDate.date() == QDate::currentDate();
+    QString date =
+        isToday ? m_locale.toString(info.authorDate.time(), QLocale::NarrowFormat) : m_locale.toString(info.authorDate.date(), QLocale::NarrowFormat);
+
     QString text = info.summary.isEmpty()
-        ? i18nc("git-blame information \"author: date \"", " %1: %2 ", info.authorName, m_locale.toString(info.authorDate, QLocale::NarrowFormat))
-        : i18nc("git-blame information \"author: date: commit title \"",
-                " %1: %2: %3 ",
-                info.authorName,
-                m_locale.toString(info.authorDate, QLocale::NarrowFormat),
-                QString::fromUtf8(info.summary));
+        ? i18nc("git-blame information \"author: date \"", " %1: %2 ", info.authorName, date)
+        : i18nc("git-blame information \"author: date: commit title \"", " %1: %2: %3 ", info.authorName, date, QString::fromUtf8(info.summary));
     QRect rectangle{0, 0, fm.horizontalAdvance(text), note.lineHeight()};
 
     auto editor = KTextEditor::Editor::instance();
@@ -112,6 +116,10 @@ void GitBlameInlineNoteProvider::inlineNoteActivated(const KTextEditor::InlineNo
         int lineNr = note.position().line();
         const CommitInfo &info = m_pluginView->blameInfo(lineNr);
 
+        if (isUncomittedLine(info.hash)) {
+            return;
+        }
+
         // Hack: view->mainWindow()->view() to de-constify view
         Q_ASSERT(note.view() == m_pluginView->activeView());
         m_pluginView->showCommitInfo(QString::fromUtf8(info.hash), note.view()->mainWindow()->activeView());
@@ -299,9 +307,33 @@ static int nextBlockStart(const QByteArray &out, int from)
     return next;
 }
 
-void KateGitBlamePluginView::blameFinished(int /*exitCode*/, QProcess::ExitStatus /*exitStatus*/)
+void KateGitBlamePluginView::sendMessage(const QString &text, bool error)
 {
-    const QByteArray out = m_blameInfoProc.readAllStandardOutput();
+    QVariantMap genericMessage;
+    genericMessage.insert(QStringLiteral("type"), error ? QStringLiteral("Error") : QStringLiteral("Info"));
+    genericMessage.insert(QStringLiteral("category"), i18n("Git"));
+    genericMessage.insert(QStringLiteral("categoryIcon"), QIcon(QStringLiteral(":/icons/icons/sc-apps-git.svg")));
+    genericMessage.insert(QStringLiteral("text"), text);
+    Q_EMIT message(genericMessage);
+}
+
+void KateGitBlamePluginView::blameFinished(int exitCode, QProcess::ExitStatus exitStatus)
+{
+    if (exitCode != 0 || exitStatus != QProcess::NormalExit) {
+        const auto error = m_blameInfoProc.readAllStandardError();
+
+        // Ignore, this repo doesn't have any commits or not a repo
+        if (error.startsWith("fatal: no such ref: HEAD") || error.startsWith("fatal: not a git repository") || error.startsWith("fatal: no such path ")) {
+            return;
+        }
+
+        QString text = i18n("Git blame failed.");
+        sendMessage(text + QStringLiteral("\n") + QString::fromUtf8(error), true);
+        return;
+    }
+
+    QByteArray out = m_blameInfoProc.readAllStandardOutput();
+    out.replace("\r", ""); // KTextEditor removes all \r characters in the internal buffers
     // printf("recieved output: %d for: git %s\n", out.size(), qPrintable(m_blameInfoProc.arguments().join(QLatin1Char(' '))));
 
     /**
@@ -426,7 +458,8 @@ void KateGitBlamePluginView::blameFinished(int /*exitCode*/, QProcess::ExitStatu
 void KateGitBlamePluginView::showFinished(int exitCode, QProcess::ExitStatus exitStatus)
 {
     if (exitCode != 0 || exitStatus != QProcess::NormalExit) {
-        qWarning() << "Failed to show commit";
+        QString text = i18n("Git blame, show commit failed.");
+        sendMessage(text + QStringLiteral("\n") + QString::fromUtf8(m_showProc.readAllStandardError()), true);
         return;
     }
 
