@@ -50,13 +50,13 @@ KateSQLView::KateSQLView(KTextEditor::Plugin *plugin, KTextEditor::MainWindow *m
                                           QStringLiteral("kate_private_plugin_katesql_output"),
                                           KTextEditor::MainWindow::Bottom,
                                           QIcon::fromTheme(QStringLiteral("view-form-table")),
-                                          i18nc("@title:window", "SQL Results"));
+                                          i18nc("@title:window", "SQL"));
 
     m_schemaBrowserToolView = mw->createToolView(plugin,
                                                  QStringLiteral("kate_private_plugin_katesql_schemabrowser"),
                                                  KTextEditor::MainWindow::Left,
                                                  QIcon::fromTheme(QStringLiteral("view-list-tree")),
-                                                 i18nc("@title:window", "SQL Schema Browser"));
+                                                 i18nc("@title:window", "SQL Schema"));
 
     m_outputWidget = new KateSQLOutputWidget(m_outputToolView);
 
@@ -192,19 +192,9 @@ void KateSQLView::slotGlobalSettingsChanged()
     m_outputWidget->dataOutputWidget()->model()->readConfig();
 }
 
-void KateSQLView::readSessionConfig(KConfigBase *config, const QString &groupPrefix)
+void KateSQLView::readSessionConfig(KConfigGroup const &group)
 {
-    KConfigGroup globalConfig(KSharedConfig::openConfig(), "KateSQLPlugin");
-
-    bool saveConnections = globalConfig.readEntry("SaveConnections", true);
-
-    if (!saveConnections) {
-        return;
-    }
-
-    KConfigGroup group(config, groupPrefix + QLatin1String(":connections"));
-
-    m_manager->loadConnections(&group);
+    m_manager->loadConnections(group);
 
     QString lastConnection = group.readEntry("LastUsed");
 
@@ -213,10 +203,8 @@ void KateSQLView::readSessionConfig(KConfigBase *config, const QString &groupPre
     }
 }
 
-void KateSQLView::writeSessionConfig(KConfigBase *config, const QString &groupPrefix)
+void KateSQLView::writeSessionConfig(KConfigGroup &group)
 {
-    KConfigGroup group(config, groupPrefix + QLatin1String(":connections"));
-
     group.deleteGroup();
 
     KConfigGroup globalConfig(KSharedConfig::openConfig(), "KateSQLPlugin");
@@ -227,8 +215,7 @@ void KateSQLView::writeSessionConfig(KConfigBase *config, const QString &groupPr
 
         group.writeEntry("LastUsed", m_connectionsComboBox->currentText());
     }
-
-    config->sync();
+    group.config()->sync();
 }
 
 void KateSQLView::slotConnectionCreate()
@@ -247,7 +234,7 @@ void KateSQLView::slotConnectionCreate()
 
     m_manager->createConnection(c);
 
-    if (m_manager->storeCredentials(c) != 0) {
+    if (m_manager->storeCredentials(c) != SQLManager::K_WALLET_CONNECTION_SUCCESSFUL) {
         qDebug() << "Connection credentials not saved";
     }
 }
@@ -274,7 +261,7 @@ void KateSQLView::slotConnectionEdit()
     m_manager->removeConnection(previousName);
     m_manager->createConnection(c);
 
-    if (m_manager->storeCredentials(c) != 0) {
+    if (m_manager->storeCredentials(c) != SQLManager::K_WALLET_CONNECTION_SUCCESSFUL) {
         qDebug() << "Connection credentials not saved";
     }
 }
