@@ -25,48 +25,52 @@
 #include <QPlainTextDocumentLayout>
 #include <utility>
 
-KateProject::KateProject(QThreadPool &threadPool, KateProjectPlugin *plugin)
-    : m_notesDocument(nullptr)
-    , m_untrackedDocumentsRoot(nullptr)
-    , m_threadPool(threadPool)
+KateProject::KateProject(QThreadPool &threadPool, KateProjectPlugin *plugin, const QString &fileName)
+    : m_threadPool(threadPool)
     , m_plugin(plugin)
+    , m_fileBacked(true)
+    , m_fileName(QFileInfo(fileName).canonicalFilePath())
+    , m_baseDir(QFileInfo(fileName).canonicalPath())
 {
+    // if canonicalFilePath already returned empty string, no need to try to load this
+    if (m_fileName.isEmpty()) {
+        return;
+    }
+
+    // ensure we get notified for project file changes
+    connect(&m_plugin->fileWatcher(), &QFileSystemWatcher::fileChanged, this, &KateProject::slotFileChanged);
+    m_plugin->fileWatcher().addPath(m_fileName);
+
+    // try to load the project map from our file, will start worker thread, too
+    reload();
+}
+
+KateProject::KateProject(QThreadPool &threadPool, KateProjectPlugin *plugin, const QVariantMap &globalProject, const QString &directory)
+    : m_threadPool(threadPool)
+    , m_plugin(plugin)
+    , m_fileBacked(false)
+    , m_fileName(QDir(QDir(directory).canonicalPath()).filePath(QStringLiteral(".kateproject")))
+    , m_baseDir(QDir(directory).canonicalPath())
+    , m_globalProject(globalProject)
+{
+    // try to load the project map, will start worker thread, too
+    load(globalProject);
 }
 
 KateProject::~KateProject()
 {
     saveNotesDocument();
-}
 
-bool KateProject::loadFromFile(const QString &fileName)
-{
-    /**
-     * bail out if already fileName set!
-     */
-    if (!m_fileName.isEmpty()) {
-        return false;
+    // stop watching if we have some real project file
+    if (m_fileBacked && !m_fileName.isEmpty()) {
+        m_plugin->fileWatcher().removePath(m_fileName);
     }
-
-    /**
-     * set new filename and base directory
-     */
-    m_fileName = fileName;
-    m_baseDir = QFileInfo(m_fileName).canonicalPath();
-
-    /**
-     * trigger reload
-     */
-    return reload();
 }
 
 bool KateProject::reload(bool force)
 {
-    QVariantMap map = readProjectFile();
-
-    if (map.isEmpty()) {
-        m_fileLastModified = QDateTime();
-    } else {
-        m_fileLastModified = QFileInfo(m_fileName).lastModified();
+    const QVariantMap map = readProjectFile();
+    if (!map.isEmpty()) {
         m_globalProject = map;
     }
 
@@ -101,8 +105,11 @@ void KateProject::removeFile(const QString &file)
  */
 QJsonDocument KateProject::readJSONFile(const QString &fileName)
 {
-    QFile file(fileName);
+    if (fileName.isEmpty()) {
+        return QJsonDocument();
+    }
 
+    QFile file(fileName);
     if (!file.exists() || !file.open(QFile::ReadOnly)) {
         return QJsonDocument();
     }
@@ -123,8 +130,13 @@ QJsonDocument KateProject::readJSONFile(const QString &fileName)
 
 QVariantMap KateProject::readProjectFile() const
 {
-    QJsonDocument project(readJSONFile(m_fileName));
+    // not file back => will not work
+    if (!m_fileBacked) {
+        return QVariantMap();
+    }
+
     // bail out on error
+    QJsonDocument project(readJSONFile(m_fileName));
     if (project.isNull()) {
         return QVariantMap();
     }
@@ -167,14 +179,6 @@ QVariantMap KateProject::readProjectFile() const
     return project.toVariant().toMap();
 }
 
-bool KateProject::loadFromData(const QVariantMap &globalProject, const QString &directory)
-{
-    m_baseDir = directory;
-    m_fileName = QDir(directory).filePath(QStringLiteral(".kateproject"));
-    m_globalProject = globalProject;
-    return load(globalProject);
-}
-
 bool KateProject::load(const QVariantMap &globalProject, bool force)
 {
     /**
@@ -186,9 +190,11 @@ bool KateProject::load(const QVariantMap &globalProject, bool force)
 
     /**
      * support out-of-source project files
+     * ensure we handle relative paths properly => relative to the potential invented .kateproject file name
      */
-    if (!globalProject[QStringLiteral("directory")].toString().isEmpty()) {
-        m_baseDir = QFileInfo(globalProject[QStringLiteral("directory")].toString()).canonicalFilePath();
+    const auto baseDir = globalProject[QStringLiteral("directory")].toString();
+    if (!baseDir.isEmpty()) {
+        m_baseDir = QFileInfo(QFileInfo(m_fileName).dir(), baseDir).canonicalFilePath();
     }
 
     /**
@@ -491,3 +497,10 @@ void KateProject::unregisterUntrackedItem(const KateProjectItem *item)
         m_untrackedDocumentsRoot = nullptr;
     }
 }
+
+void KateProject::slotFileChanged(const QString &file)
+{
+    if (file == m_fileName) {
+        reload();
+    }
+}
