@@ -48,10 +48,12 @@ const QString ProjectFileName = QStringLiteral(".kateproject");
 const QString GitFolderName = QStringLiteral(".git");
 const QString SubversionFolderName = QStringLiteral(".svn");
 const QString MercurialFolderName = QStringLiteral(".hg");
+const QString FossilCheckoutFileName = QStringLiteral(".fslckout");
 
 const QString GitConfig = QStringLiteral("git");
 const QString SubversionConfig = QStringLiteral("subversion");
 const QString MercurialConfig = QStringLiteral("mercurial");
+const QString FossilConfig = QStringLiteral("fossil");
 
 const QStringList DefaultConfig = QStringList() << GitConfig << SubversionConfig << MercurialConfig;
 }
@@ -65,7 +67,6 @@ KateProjectPlugin::KateProjectPlugin(QObject *parent, const QList<QVariant> &)
     qRegisterMetaType<KateProjectSharedProjectIndex>("KateProjectSharedProjectIndex");
 
     connect(KTextEditor::Editor::instance()->application(), &KTextEditor::Application::documentCreated, this, &KateProjectPlugin::slotDocumentCreated);
-    connect(&m_fileWatcher, &QFileSystemWatcher::directoryChanged, this, &KateProjectPlugin::slotDirectoryChanged);
 
     // read configuration prior to cwd project setup below
     readConfig();
@@ -119,7 +120,6 @@ KateProjectPlugin::~KateProjectPlugin()
     unregisterVariables();
 
     for (KateProject *project : qAsConst(m_projects)) {
-        m_fileWatcher.removePath(QFileInfo(project->fileName()).canonicalPath());
         delete project;
     }
     m_projects.clear();
@@ -145,14 +145,13 @@ KTextEditor::ConfigPage *KateProjectPlugin::configPage(int number, QWidget *pare
 
 KateProject *KateProjectPlugin::createProjectForFileName(const QString &fileName)
 {
-    KateProject *project = new KateProject(m_threadPool, this);
-    if (!project->loadFromFile(fileName)) {
+    KateProject *project = new KateProject(m_threadPool, this, fileName);
+    if (!project->isValid()) {
         delete project;
         return nullptr;
     }
 
     m_projects.append(project);
-    m_fileWatcher.addPath(QFileInfo(fileName).canonicalPath());
     Q_EMIT projectCreated(project);
     return project;
 }
@@ -206,7 +205,7 @@ KateProject *KateProjectPlugin::projectForDir(QDir dir, bool userSpecified)
     for (const QString &dir : directoryStack) {
         // try to invent project based on version control stuff
         KateProject *project = nullptr;
-        if ((project = detectGit(dir)) || (project = detectSubversion(dir)) || (project = detectMercurial(dir))) {
+        if ((project = detectGit(dir)) || (project = detectSubversion(dir)) || (project = detectMercurial(dir)) || (project = detectFossil(dir))) {
             return project;
         }
     }
@@ -247,7 +246,6 @@ bool KateProjectPlugin::closeProject(KateProject *project)
 
     Q_EMIT pluginViewProjectClosing(project);
     if (m_projects.removeOne(project)) {
-        m_fileWatcher.removePath(QFileInfo(project->fileName()).canonicalPath());
         delete project;
         return true;
     }
@@ -304,20 +302,6 @@ void KateProjectPlugin::slotDocumentUrlChanged(KTextEditor::Document *document)
     }
 }
 
-void KateProjectPlugin::slotDirectoryChanged(const QString &path)
-{
-    QString fileName = QDir(path).filePath(ProjectFileName);
-    for (KateProject *project : qAsConst(m_projects)) {
-        if (project->fileName() == fileName) {
-            QDateTime lastModified = QFileInfo(fileName).lastModified();
-            if (project->fileLastModified().isNull() || (lastModified > project->fileLastModified())) {
-                project->reload();
-            }
-            break;
-        }
-    }
-}
-
 KateProject *KateProjectPlugin::detectGit(const QDir &dir)
 {
     // allow .git as dir and file (file for git worktree stuff, https://git-scm.com/docs/git-worktree)
@@ -346,6 +330,15 @@ KateProject *KateProjectPlugin::detectMercurial(const QDir &dir)
     return nullptr;
 }
 
+KateProject *KateProjectPlugin::detectFossil(const QDir &dir)
+{
+    if (m_autoFossil && dir.exists(FossilCheckoutFileName) && QFileInfo(dir, FossilCheckoutFileName).isReadable()) {
+        return createProjectForRepository(QStringLiteral("fossil"), dir);
+    }
+
+    return nullptr;
+}
+
 KateProject *KateProjectPlugin::createProjectForRepository(const QString &type, const QDir &dir)
 {
     QVariantMap cnf, files;
@@ -353,8 +346,7 @@ KateProject *KateProjectPlugin::createProjectForRepository(const QString &type,
     cnf[QStringLiteral("name")] = dir.dirName();
     cnf[QStringLiteral("files")] = (QVariantList() << files);
 
-    KateProject *project = new KateProject(m_threadPool, this);
-    project->loadFromData(cnf, dir.canonicalPath());
+    KateProject *project = new KateProject(m_threadPool, this, cnf, dir.canonicalPath());
 
     m_projects.append(project);
 
@@ -369,8 +361,7 @@ KateProject *KateProjectPlugin::createProjectForDirectory(const QDir &dir)
     cnf[QStringLiteral("name")] = dir.dirName();
     cnf[QStringLiteral("files")] = (QVariantList() << files);
 
-    KateProject *project = new KateProject(m_threadPool, this);
-    project->loadFromData(cnf, dir.canonicalPath());
+    KateProject *project = new KateProject(m_threadPool, this, cnf, dir.canonicalPath());
 
     m_projects.append(project);
 
@@ -378,11 +369,23 @@ KateProject *KateProjectPlugin::createProjectForDirectory(const QDir &dir)
     return project;
 }
 
-void KateProjectPlugin::setAutoRepository(bool onGit, bool onSubversion, bool onMercurial)
+
+KateProject *KateProjectPlugin::createProjectForDirectory(const QDir &dir, const QVariantMap &projectMap)
+{
+    KateProject *project = new KateProject(m_threadPool, this, projectMap, dir.canonicalPath());
+
+    m_projects.append(project);
+
+    Q_EMIT projectCreated(project);
+    return project;
+}
+
+void KateProjectPlugin::setAutoRepository(bool onGit, bool onSubversion, bool onMercurial, bool onFossil)
 {
     m_autoGit = onGit;
     m_autoSubversion = onSubversion;
     m_autoMercurial = onMercurial;
+    m_autoFossil = onFossil;
     writeConfig();
 }
 
@@ -401,6 +404,11 @@ bool KateProjectPlugin::autoMercurial() const
     return m_autoMercurial;
 }
 
+bool KateProjectPlugin::autoFossil() const
+{
+    return m_autoFossil;
+}
+
 void KateProjectPlugin::setIndex(bool enabled, const QUrl &directory)
 {
     m_indexEnabled = enabled;
@@ -434,7 +442,7 @@ void KateProjectPlugin::setGitStatusShowNumStat(bool show)
     writeConfig();
 }
 
-bool KateProjectPlugin::showGitStatusWithNumStat()
+bool KateProjectPlugin::showGitStatusWithNumStat() const
 {
     return m_gitNumStat;
 }
@@ -476,6 +484,7 @@ void KateProjectPlugin::readConfig()
     m_autoGit = autorepository.contains(GitConfig);
     m_autoSubversion = autorepository.contains(SubversionConfig);
     m_autoMercurial = autorepository.contains(MercurialConfig);
+    m_autoFossil = autorepository.contains(FossilConfig);
 
     m_indexEnabled = config.readEntry("index", false);
     m_indexDirectory = config.readEntry("indexDirectory", QUrl());
@@ -507,6 +516,10 @@ void KateProjectPlugin::writeConfig()
         repos << MercurialConfig;
     }
 
+    if (m_autoFossil) {
+        repos << FossilConfig;
+    }
+
     config.writeEntry("autorepository", repos);
 
     config.writeEntry("index", m_indexEnabled);
@@ -565,10 +578,51 @@ void KateProjectPlugin::registerVariables()
                                       return QDir::toNativeSeparators(QDir(kateProject->baseDir()).absolutePath());
                                   });
 }
-
 void KateProjectPlugin::unregisterVariables()
 {
     auto editor = KTextEditor::Editor::instance();
     editor->unregisterVariableMatch(QStringLiteral("Project:Path"));
     editor->unregisterVariableMatch(QStringLiteral("Project:NativePath"));
 }
+
+void KateProjectPlugin::readSessionConfig(const KConfigGroup &config)
+{
+    QByteArray buffer;
+    QVariantMap projectMap;
+    const QVariantList projectList = config.readEntry("projects", QVariantList());
+
+    for (const QVariant &project : projectList) {
+        buffer = project.toByteArray();
+
+        {
+            QDataStream stream(&buffer, QIODevice::ReadOnly);
+            stream >> projectMap;
+        }
+
+        createProjectForDirectory(QDir(projectMap[QStringLiteral("path")].toString()), projectMap[QStringLiteral("data")].toMap());
+    }
+}
+
+void KateProjectPlugin::writeSessionConfig(KConfigGroup &config)
+{
+    QVariantList projectList;
+    QByteArray buffer;
+
+    for (KateProject *project : projects()) {
+        if (!project->isFileBacked()) {
+            QVariantMap sMap;
+
+            sMap[QStringLiteral("data")] = project->projectMap();
+            sMap[QStringLiteral("path")] = project->baseDir();
+
+            {
+                QDataStream stream(&buffer, QIODevice::WriteOnly);
+                stream << sMap;
+            }
+
+            projectList.push_back(buffer);
+        }
+    }
+
+    config.writeEntry("projects", projectList);
+}
