@@ -13,11 +13,11 @@
 #include <QThreadPool>
 
 #include <KTextEditor/Plugin>
+#include <KTextEditor/SessionConfigInterface>
+#include <KXMLGUIClient>
 #include <ktexteditor/document.h>
 #include <ktexteditor/mainwindow.h>
 
-#include <KXMLGUIClient>
-
 #include "kateprojectcompletion.h"
 
 class KateProject;
@@ -29,9 +29,10 @@ enum class ClickAction : uint8_t {
     StageUnstage,
 };
 
-class KateProjectPlugin : public KTextEditor::Plugin
+class KateProjectPlugin : public KTextEditor::Plugin, public KTextEditor::SessionConfigInterface
 {
     Q_OBJECT
+    Q_INTERFACES(KTextEditor::SessionConfigInterface)
 
 public:
     explicit KateProjectPlugin(QObject *parent = nullptr, const QList<QVariant> & = QList<QVariant>());
@@ -105,10 +106,11 @@ public:
         return m_document2Project.value(document);
     }
 
-    void setAutoRepository(bool onGit, bool onSubversion, bool onMercurial);
+    void setAutoRepository(bool onGit, bool onSubversion, bool onMercurial, bool onFossil);
     bool autoGit() const;
     bool autoSubversion() const;
     bool autoMercurial() const;
+    bool autoFossil() const;
 
     void setIndex(bool enabled, const QUrl &directory);
     bool getIndexEnabled() const;
@@ -119,7 +121,7 @@ public:
     bool multiProjectGoto() const;
 
     void setGitStatusShowNumStat(bool show);
-    bool showGitStatusWithNumStat();
+    bool showGitStatusWithNumStat() const;
 
     void setSingleClickAction(ClickAction cb);
     ClickAction singleClickAcion();
@@ -127,6 +129,15 @@ public:
     void setDoubleClickAction(ClickAction cb);
     ClickAction doubleClickAcion();
 
+    /**
+     * filesystem watcher to keep track of all project files
+     * and auto-reload
+     */
+    QFileSystemWatcher &fileWatcher()
+    {
+        return m_fileWatcher;
+    }
+
 Q_SIGNALS:
 
     /**
@@ -171,24 +182,23 @@ public Q_SLOTS:
      */
     void slotDocumentUrlChanged(KTextEditor::Document *document);
 
-    /**
-     * did some project file change?
-     * @param path name of directory that did change
-     */
-    void slotDirectoryChanged(const QString &path);
-
 private:
     KateProject *createProjectForRepository(const QString &type, const QDir &dir);
     KateProject *createProjectForDirectory(const QDir &dir);
+    KateProject *createProjectForDirectory(const QDir &dir, const QVariantMap &projectMap);
     KateProject *detectGit(const QDir &dir);
     KateProject *detectSubversion(const QDir &dir);
     KateProject *detectMercurial(const QDir &dir);
+    KateProject *detectFossil(const QDir &dir);
+
+    void readSessionConfig(const KConfigGroup &config) override;
+    void writeSessionConfig(KConfigGroup &config) override;
 
     void readConfig();
     void writeConfig();
 
-    void registerVariables();
-    void unregisterVariables();
+    static void registerVariables();
+    static void unregisterVariables();
 
 private:
     /**
@@ -216,6 +226,7 @@ private:
     bool m_autoGit = true;
     bool m_autoSubversion = true;
     bool m_autoMercurial = true;
+    bool m_autoFossil = true;
 
     // indexing is expensive, default off
     bool m_indexEnabled = false;
