@@ -72,7 +72,7 @@ KateProjectPluginView::KateProjectPluginView(KateProjectPlugin *plugin, KTextEdi
                                                   QStringLiteral("kateprojectinfo"),
                                                   KTextEditor::MainWindow::Bottom,
                                                   QIcon::fromTheme(QStringLiteral("view-choose")),
-                                                  i18n("Current Project"));
+                                                  i18n("Project"));
 
     /**
      * create the combo + buttons for the toolViews + stacked widgets
@@ -733,6 +733,7 @@ void KateProjectPluginView::openDirectoryOrProject()
         if (index >= 0) {
             m_projectsCombo->setCurrentIndex(index);
         }
+        mainWindow()->showToolView(m_toolView);
     }
 }
 
@@ -772,6 +773,20 @@ void KateProjectPluginView::openTerminal(const QString &dirPath, KateProject *pr
     }
 }
 
+QString KateProjectPluginView::projectBaseDirForUrl(const QUrl &url)
+{
+    const auto projects = m_plugin->projects();
+    QString ret;
+    for (auto *project : projects) {
+        const QString dir = project->baseDir();
+        if (url.toLocalFile().startsWith(dir)) {
+            // prefer the longer base path
+            ret = ret.length() < dir.length() ? dir : ret;
+        }
+    }
+    return ret;
+}
+
 void KateProjectPluginView::updateActions()
 {
     // currently some project active?
