@@ -136,6 +136,11 @@ public:
      */
     void openTerminal(const QString &dirPath, KateProject *project);
 
+    /**
+     * Returns project base dir for provided url
+     */
+    Q_INVOKABLE QString projectBaseDirForUrl(const QUrl &url);
+
 public Q_SLOTS:
     /**
      * Create views for given project.
@@ -300,7 +305,7 @@ private Q_SLOTS:
     /**
      * Show projects To-Dos and Fix-mes
      */
-    void showProjectTodos();
+    static void showProjectTodos();
 
     /**
      * Enable/disable project actions
