@@ -18,6 +18,7 @@
 #include <QRegularExpression>
 #include <QSet>
 #include <QSettings>
+#include <QStandardPaths>
 #include <QThread>
 #include <QTime>
 #include <QtConcurrent>
@@ -380,6 +381,10 @@ QVector<QString> KateProjectWorker::findFiles(const QDir &dir, const QVariantMap
         return filesFromDarcs(dir, recursive);
     }
 
+    if (filesEntry[QStringLiteral("fossil")].toBool()) {
+        return filesFromFossil(dir, recursive);
+    }
+
     /**
      * if we arrive here, we have some manual specification of files, no VCS
      */
@@ -437,15 +442,17 @@ QVector<QString> KateProjectWorker::filesFromGit(const QDir &dir, bool recursive
                                            QStringLiteral(".")};
 
     // ls-files + ls-files untracked
-    return gitFiles(dir, recursive, lsFilesArgs) << gitFiles(dir, recursive, lsFilesUntrackedArgs);
+    return gitFiles(dir, recursive, lsFilesArgs, false) << gitFiles(dir, recursive, lsFilesUntrackedArgs, true);
 }
 
-QVector<QString> KateProjectWorker::gitFiles(const QDir &dir, bool recursive, const QStringList &args)
+QVector<QString> KateProjectWorker::gitFiles(const QDir &dir, bool recursive, const QStringList &args, bool ignoreBinaryFiles)
 {
-    QProcess git;
-    setupGitProcess(git, dir.absolutePath(), args);
-    git.start(QProcess::ReadOnly);
     QVector<QString> files;
+    QProcess git;
+    if (!setupGitProcess(git, dir.absolutePath(), args)) {
+        return files;
+    }
+    git.start(QProcess::ReadOnly);
     if (!git.waitForStarted() || !git.waitForFinished(-1)) {
         return files;
     }
@@ -461,18 +468,35 @@ QVector<QString> KateProjectWorker::gitFiles(const QDir &dir, bool recursive, co
         }
         files.append(QString::fromUtf8(byteArray));
     }
+
+    // Filter out binary files
+    if (ignoreBinaryFiles) {
+        QtConcurrent::blockingMap(files, [dir](const QString &file) {
+            const QMimeType mimeType = QMimeDatabase().mimeTypeForFile(dir.filePath(file));
+            if (!mimeType.inherits(QStringLiteral("text/plain"))) {
+                return QString();
+            }
+            return file;
+        });
+    }
+
     return files;
 }
 
 QVector<QString> KateProjectWorker::filesFromMercurial(const QDir &dir, bool recursive)
 {
+    // only use version control from PATH
     QVector<QString> files;
+    static const auto fullExecutablePath = QStandardPaths::findExecutable(QStringLiteral("hg"));
+    if (fullExecutablePath.isEmpty()) {
+        return files;
+    }
 
     QProcess hg;
     hg.setWorkingDirectory(dir.absolutePath());
     QStringList args;
     args << QStringLiteral("manifest") << QStringLiteral(".");
-    hg.start(QStringLiteral("hg"), args, QProcess::ReadOnly);
+    hg.start(fullExecutablePath, args, QProcess::ReadOnly);
     if (!hg.waitForStarted() || !hg.waitForFinished(-1)) {
         return files;
     }
@@ -493,7 +517,12 @@ QVector<QString> KateProjectWorker::filesFromMercurial(const QDir &dir, bool rec
 
 QVector<QString> KateProjectWorker::filesFromSubversion(const QDir &dir, bool recursive)
 {
+    // only use version control from PATH
     QVector<QString> files;
+    static const auto fullExecutablePath = QStandardPaths::findExecutable(QStringLiteral("svn"));
+    if (fullExecutablePath.isEmpty()) {
+        return files;
+    }
 
     QProcess svn;
     svn.setWorkingDirectory(dir.absolutePath());
@@ -504,7 +533,7 @@ QVector<QString> KateProjectWorker::filesFromSubversion(const QDir &dir, bool re
     } else {
         args << QStringLiteral("--depth=files");
     }
-    svn.start(QStringLiteral("svn"), args, QProcess::ReadOnly);
+    svn.start(fullExecutablePath, args, QProcess::ReadOnly);
     if (!svn.waitForStarted() || !svn.waitForFinished(-1)) {
         return files;
     }
@@ -555,18 +584,21 @@ QVector<QString> KateProjectWorker::filesFromSubversion(const QDir &dir, bool re
 
 QVector<QString> KateProjectWorker::filesFromDarcs(const QDir &dir, bool recursive)
 {
+    // only use version control from PATH
     QVector<QString> files;
+    static const auto fullExecutablePath = QStandardPaths::findExecutable(QStringLiteral("darcs"));
+    if (fullExecutablePath.isEmpty()) {
+        return files;
+    }
 
-    const QString cmd = QStringLiteral("darcs");
     QString root;
-
     {
         QProcess darcs;
         darcs.setWorkingDirectory(dir.absolutePath());
         QStringList args;
         args << QStringLiteral("list") << QStringLiteral("repo");
 
-        darcs.start(cmd, args, QProcess::ReadOnly);
+        darcs.start(fullExecutablePath, args, QProcess::ReadOnly);
 
         if (!darcs.waitForStarted() || !darcs.waitForFinished(-1)) {
             return files;
@@ -590,7 +622,7 @@ QVector<QString> KateProjectWorker::filesFromDarcs(const QDir &dir, bool recursi
         darcs.setWorkingDirectory(dir.absolutePath());
         args << QStringLiteral("list") << QStringLiteral("files") << QStringLiteral("--no-directories") << QStringLiteral("--pending");
 
-        darcs.start(cmd, args, QProcess::ReadOnly);
+        darcs.start(fullExecutablePath, args, QProcess::ReadOnly);
 
         if (!darcs.waitForStarted() || !darcs.waitForFinished(-1)) {
             return files;
@@ -613,6 +645,38 @@ QVector<QString> KateProjectWorker::filesFromDarcs(const QDir &dir, bool recursi
     return files;
 }
 
+QVector<QString> KateProjectWorker::filesFromFossil(const QDir &dir, bool recursive)
+{
+    // only use version control from PATH
+    QVector<QString> files;
+    static const auto fullExecutablePath = QStandardPaths::findExecutable(QStringLiteral("fossil"));
+    if (fullExecutablePath.isEmpty()) {
+        return files;
+    }
+
+    QProcess fossil;
+    fossil.setWorkingDirectory(dir.absolutePath());
+    QStringList args;
+    args << QStringLiteral("ls");
+    fossil.start(fullExecutablePath, args, QProcess::ReadOnly);
+    if (!fossil.waitForStarted() || !fossil.waitForFinished(-1)) {
+        return files;
+    }
+
+    const QStringList relFiles = QString::fromLocal8Bit(fossil.readAllStandardOutput()).split(QRegularExpression(QStringLiteral("[\n\r]")), Qt::SkipEmptyParts);
+
+    files.reserve(relFiles.size());
+    for (const QString &relFile : relFiles) {
+        if (!recursive && (relFile.indexOf(QLatin1Char('/')) != -1)) {
+            continue;
+        }
+
+        files.append(relFile);
+    }
+
+    return files;
+}
+
 QVector<QString> KateProjectWorker::filesFromDirectory(const QDir &_dir, bool recursive, const QStringList &filters)
 {
     /**
@@ -635,13 +699,25 @@ QVector<QString> KateProjectWorker::filesFromDirectory(const QDir &_dir, bool re
     /**
      * create iterator and collect all files
      */
-    QVector<QString> files;
     QDirIterator dirIterator(dir, flags);
     const QString dirPath = dir.path() + QLatin1Char('/');
+
+    std::vector<QFileInfo> fileInfos;
+
     while (dirIterator.hasNext()) {
         dirIterator.next();
-        // make it relative path
-        files.append(dirIterator.filePath().remove(dirPath));
+        fileInfos.push_back(dirIterator.fileInfo());
     }
-    return files;
+
+    // Filter out binary files
+    std::function<QString(const QFileInfo &)> func = [dirPath](const QFileInfo &fi) {
+        // We only accept plainText files
+        const QMimeType mimeType = QMimeDatabase().mimeTypeForFile(fi);
+        if (!mimeType.inherits(QStringLiteral("text/plain"))) {
+            return QString();
+        }
+        return fi.filePath().remove(dirPath);
+    };
+
+    return QtConcurrent::blockingMapped<QVector<QString>>(fileInfos, func);
 }
