@@ -52,11 +52,9 @@
 #include <KToggleFullScreenAction>
 #include <KToolBar>
 #include <KWindowConfig>
-#include <KWindowSystem>
 #include <KXMLGUIFactory>
 
 #include <QApplication>
-#include <QDesktopWidget>
 #include <QDir>
 #include <QFontDatabase>
 #include <QList>
@@ -64,6 +62,7 @@
 #include <QMenuBar>
 #include <QMimeData>
 #include <QMimeDatabase>
+#include <QScreen>
 #include <QStyle>
 #include <QTimer>
 #include <QToolButton>
@@ -204,10 +203,8 @@ KateMainWindow::~KateMainWindow()
 
 QSize KateMainWindow::sizeHint() const
 {
-    /**
-     * have some useful size hint, else we have mini windows per default
-     */
-    return (QSize(640, 480).expandedTo(minimumSizeHint()));
+    // ensure some proper sizing per default
+    return (QSize(800, 600).expandedTo(minimumSizeHint())).expandedTo(screen()->availableSize() * 0.6).boundedTo(screen()->availableSize());
 }
 
 void KateMainWindow::setupImportantActions()
@@ -226,6 +223,12 @@ void KateMainWindow::setupImportantActions()
     connect(m_paShowPath, SIGNAL(toggled(bool)), this, SLOT(updateCaption()));
     m_paShowPath->setWhatsThis(i18n("Show the complete document path in the window caption"));
 
+    m_paShowUrlNavBar = new KToggleAction(i18n("Show Navigation Bar"), this);
+    actionCollection()->addAction(QStringLiteral("settings_show_url_nav_bar"), m_paShowUrlNavBar);
+    connect(m_paShowUrlNavBar, &QAction::toggled, this, [this](bool v) {
+        m_viewManager->setShowUrlNavBar(v);
+    });
+
     // Load themes
     actionCollection()->addAction(QStringLiteral("colorscheme_menu"), new KateColorSchemeChooser(actionCollection()));
 
@@ -617,7 +620,8 @@ void KateMainWindow::reloadXmlGui()
 {
     for (KTextEditor::Document *doc : KateApp::self()->documentManager()->documentList()) {
         doc->reloadXML();
-        for (KTextEditor::View *view : doc->views()) {
+        const auto views = doc->views();
+        for (KTextEditor::View *view : views) {
             view->reloadXML();
         }
     }
@@ -664,10 +668,12 @@ void KateMainWindow::readOptions()
     m_paShowStatusBar->setChecked(generalGroup.readEntry("Show Status Bar", true));
     m_paShowMenuBar->setChecked(generalGroup.readEntry("Show Menu Bar", true));
     m_paShowTabBar->setChecked(generalGroup.readEntry("Show Tab Bar", true));
+    m_paShowUrlNavBar->setChecked(generalGroup.readEntry("Show Url Nav Bar", true));
 
     // emit signal to hide/show statusbars
     toggleShowStatusBar();
     toggleShowTabBar();
+    m_viewManager->setShowUrlNavBar(m_paShowUrlNavBar->isChecked());
 }
 
 void KateMainWindow::saveOptions()
@@ -684,6 +690,7 @@ void KateMainWindow::saveOptions()
     generalGroup.writeEntry("Show Status Bar", m_paShowStatusBar->isChecked());
     generalGroup.writeEntry("Show Menu Bar", m_paShowMenuBar->isChecked());
     generalGroup.writeEntry("Show Tab Bar", m_paShowTabBar->isChecked());
+    generalGroup.writeEntry("Show Url Nav Bar", m_paShowUrlNavBar->isChecked());
 }
 
 void KateMainWindow::toggleShowMenuBar(bool showMessage)
@@ -712,7 +719,8 @@ void KateMainWindow::toggleShowMenuBar(bool showMessage)
 
 void KateMainWindow::toggleShowStatusBar()
 {
-    Q_EMIT statusBarToggled();
+    // just hide or show the status bar stack
+    statusBarStackedWidget()->setVisible(showStatusBar());
 }
 
 bool KateMainWindow::showStatusBar()
@@ -1028,6 +1036,7 @@ void KateMainWindow::updateCaption(KTextEditor::Document *doc)
 {
     if (!m_viewManager->activeView()) {
         setCaption(QString(), false);
+        setWindowFilePath(QString());
         return;
     }
 
@@ -1048,6 +1057,8 @@ void KateMainWindow::updateCaption(KTextEditor::Document *doc)
         }
     }
 
+    setWindowFilePath(m_viewManager->activeView()->document()->url().toString(QUrl::PreferLocalFile));
+
     QString sessName = KateApp::self()->sessionManager()->activeSession()->name();
     if (!sessName.isEmpty()) {
         sessName = QStringLiteral("%1: ").arg(sessName);
@@ -1135,19 +1146,51 @@ void KateMainWindow::restoreWindowConfig(const KConfigGroup &config)
 
 void KateMainWindow::slotUpdateBottomViewBar()
 {
-    // qCDebug(LOG_KATE)<<"slotUpdateHorizontalViewBar()"<<endl;
+    // get active view if any, if none => just hide bar
     KTextEditor::View *view = m_viewManager->activeView();
-    BarState bs = m_bottomViewBarMapping[view];
-    if (bs.bar() && bs.state()) {
+    if (!view) {
+        if (auto wid = m_bottomContainerStack->currentWidget()) {
+            wid->hide();
+        }
+        m_bottomViewBarContainer->hide();
+        return;
+    }
+
+    // get bar state, we must have a bar widget here, KateView always creates one!
+    BarState &bs = m_bottomViewBarMapping[view];
+    Q_ASSERT(bs.bar());
+
+    // extract statusbar if not already done
+    if (!bs.statusBar()) {
+        // we search for the status bar by class, this MUST work, we ensure that it is enabled in the view
+        const auto widgets = bs.bar()->findChildren<QWidget *>(QString(), Qt::FindChildrenRecursively);
+        for (auto *w : widgets) {
+            if (w && w->metaObject()->className() == QByteArrayLiteral("KateStatusBar")) {
+                bs.setStatusBar(w);
+            }
+        }
+        Q_ASSERT(bs.statusBar());
+
+        // ensure we don't mess up the vertical sizing
+        bs.statusBar()->setSizePolicy(QSizePolicy::Expanding, QSizePolicy::Fixed);
+
+        // add the status bar to our status bar stack, there we will show/hide it
+        statusBarStackedWidget()->addWidget(bs.statusBar());
+    }
+
+    // always activate the current statusbar
+    Q_ASSERT(bs.statusBar()->parentWidget() == statusBarStackedWidget());
+    statusBarStackedWidget()->setCurrentWidget(bs.statusBar());
+
+    // hide or show the bar
+    if (bs.state()) {
         m_bottomContainerStack->setCurrentWidget(bs.bar());
         m_bottomContainerStack->currentWidget()->show();
         m_bottomViewBarContainer->show();
     } else {
-        QWidget *wid = m_bottomContainerStack->currentWidget();
-        if (wid) {
+        if (auto wid = m_bottomContainerStack->currentWidget()) {
             wid->hide();
         }
-        // qCDebug(LOG_KATE)<<wid<<"hiding container"<<endl;
         m_bottomViewBarContainer->hide();
     }
 }
@@ -1170,10 +1213,23 @@ void KateMainWindow::queueModifiedOnDisc(KTextEditor::Document *doc)
 
         s_modOnHdDialog = new KateMwModOnHdDialog(list, this);
         m_modignore = true;
-        KWindowSystem::setOnAllDesktops(s_modOnHdDialog->winId(), true);
         connect(s_modOnHdDialog, &KateMwModOnHdDialog::requestOpenDiffDocument, this, [this](const QUrl &url) {
             viewManager()->openUrl(url, QString(), true, true);
         });
+
+        // Someone modified a doc outside and now we are here
+        // but Kate isn't the active app. Delay the dialog exec
+        // otherwise it will bring us front interrupting user's
+        // work.
+        if (!qApp->activeWindow() || !hasFocus()) {
+            m_modignore = false;
+            s_modOnHdDialog->setShowOnWindowActivation(true);
+            // hopefully this shows an alert to the user in task bar
+            // that something changed in Kate
+            qApp->alert(this, 3000);
+            return;
+        }
+
         s_modOnHdDialog->exec();
         delete s_modOnHdDialog; // s_modOnHdDialog is set to 0 in destructor of KateMwModOnHdDialog (jowenn!!!)
         m_modignore = false;
@@ -1192,6 +1248,21 @@ bool KateMainWindow::event(QEvent *e)
             hideToolView(m_toolViewOutput);
         }
     }
+
+    if (e->type() == QEvent::ActivationChange) {
+        // After queueModifiedOnDisc, the app wasn't active but now it got
+        // active. Show the dialog.
+        if (s_modOnHdDialog && s_modOnHdDialog->showOnWindowActivation()) {
+            // Must set this to false, we want only one exec to happen.
+            s_modOnHdDialog->setShowOnWindowActivation(false);
+            // do it on next event loop iteration to avoid blocking here
+            QTimer::singleShot(1, this, [] {
+                s_modOnHdDialog->exec();
+                delete s_modOnHdDialog;
+            });
+        }
+    }
+
     return KateMDI::MainWindow::event(e);
 }
 
@@ -1205,6 +1276,12 @@ QObject *KateMainWindow::pluginView(const QString &name)
     return m_pluginViews.contains(plugin) ? m_pluginViews.value(plugin) : nullptr;
 }
 
+void KateMainWindow::addWidgetAsTab(QWidget *widget)
+{
+    auto vs = m_viewManager->activeViewSpace();
+    vs->addWidgetAsTab(widget);
+}
+
 void KateMainWindow::mousePressEvent(QMouseEvent *e)
 {
     switch (e->button()) {
