@@ -31,6 +31,7 @@ class KActionMenu;
 class QAction;
 class QLabel;
 class QPixmap;
+class QStackedWidget;
 class KConfigBase;
 
 namespace KTextEditor
@@ -176,6 +177,9 @@ public:
 
     void setSplitter(QSplitter *sp);
 
+    QSize minimumSizeHint() const override;
+    QSize sizeHint() const override;
+
     /**
      * Monitor resizes using the mouse and update the last size accordingly.
      * Only call this when the sidebar has siblings in the splitter (i.e. m_splitter->count() >= 2) to guarantee that resize handles exist
@@ -350,6 +354,15 @@ protected:
      */
     QWidget *centralWidget() const;
 
+    /**
+     * Status bar area stacked widget.
+     * We plug in our status bars from the KTextEditor::Views here
+     */
+    QStackedWidget *statusBarStackedWidget() const
+    {
+        return m_statusBarStackedWidget;
+    }
+
     /**
      * modifiers for existing toolviews
      */
@@ -454,6 +467,11 @@ private:
      */
     GUIClient *m_guiClient;
 
+    /**
+     * stacked widget for status bars
+     */
+    QStackedWidget *m_statusBarStackedWidget;
+
 Q_SIGNALS:
     void sigShowPluginConfigPage(KTextEditor::Plugin *configpageinterface, int id);
 };
