@@ -14,7 +14,10 @@
 #include "katemainwindow.h"
 #include "katesessionmanager.h"
 #include "kateupdatedisabler.h"
+#include "kateurlbar.h"
 #include "kateviewmanager.h"
+#include "tabmimedata.h"
+
 #include <KActionCollection>
 
 #include <KAcceleratorManager>
@@ -23,9 +26,11 @@
 
 #include <QApplication>
 #include <QClipboard>
+#include <QDataStream>
 #include <QHelpEvent>
 #include <QMenu>
 #include <QMessageBox>
+#include <QRubberBand>
 #include <QStackedWidget>
 #include <QToolButton>
 #include <QToolTip>
@@ -51,7 +56,7 @@ KateViewSpace::KateViewSpace(KateViewManager *viewManager, QWidget *parent, cons
 
     // add left <-> right history buttons
     m_historyBack = new QToolButton(this);
-    m_historyBack->setToolTip(i18n("Go Back"));
+    m_historyBack->setToolTip(i18n("Go to previous edit location"));
     m_historyBack->setIcon(QIcon::fromTheme(QStringLiteral("arrow-left")));
     m_historyBack->setAutoRaise(true);
     KAcceleratorManager::setNoAccel(m_historyBack);
@@ -64,7 +69,7 @@ KateViewSpace::KateViewSpace(KateViewManager *viewManager, QWidget *parent, cons
 
     m_historyForward = new QToolButton(this);
     m_historyForward->setIcon(QIcon::fromTheme(QStringLiteral("arrow-right")));
-    m_historyForward->setToolTip(i18n("Go Forward"));
+    m_historyForward->setToolTip(i18n("Go to next edit location"));
     m_historyForward->setAutoRaise(true);
     KAcceleratorManager::setNoAccel(m_historyForward);
     m_historyForward->installEventFilter(this); // on click, active this view space
@@ -116,6 +121,27 @@ KateViewSpace::KateViewSpace(KateViewManager *viewManager, QWidget *parent, cons
     layout->addLayout(hLayout);
     // END tab bar
 
+    m_urlBar = new KateUrlBar(this);
+
+    // like other editors, we try to re-use documents, of not modified
+    connect(m_urlBar, &KateUrlBar::openUrlRequested, this, [this](const QUrl &url, Qt::KeyboardModifiers mod) {
+        // try if reuse of view make sense
+        const bool shiftPress = mod == Qt::ShiftModifier;
+        if (!shiftPress && !KateApp::self()->documentManager()->findDocument(url)) {
+            if (auto activeView = m_viewManager->activeView(); activeView) {
+                KateDocumentInfo *info = KateApp::self()->documentManager()->documentInfo(activeView->document());
+                if (info && !info->wasDocumentEverModified) {
+                    activeView->document()->openUrl(url);
+                    return;
+                }
+            }
+        }
+
+        // default: open a new document or switch there
+        m_viewManager->openUrl(url);
+    });
+    layout->addWidget(m_urlBar);
+
     stack = new QStackedWidget(this);
     stack->setFocus();
     stack->setSizePolicy(QSizePolicy(QSizePolicy::MinimumExpanding, QSizePolicy::Expanding));
@@ -124,14 +150,23 @@ KateViewSpace::KateViewSpace(KateViewManager *viewManager, QWidget *parent, cons
     m_group.clear();
 
     // connect signal to hide/show statusbar
-    connect(m_viewManager->mainWindow(), &KateMainWindow::statusBarToggled, this, &KateViewSpace::statusBarToggled);
     connect(m_viewManager->mainWindow(), &KateMainWindow::tabBarToggled, this, &KateViewSpace::tabBarToggled);
+    connect(m_viewManager, &KateViewManager::showUrlNavBarChanged, this, &KateViewSpace::urlBarToggled);
 
-    // init the bars...
-    statusBarToggled();
+    connect(this, &KateViewSpace::viewSpaceEmptied, m_viewManager, &KateViewManager::onViewSpaceEmptied);
+
+    // we accept drops (tabs from other viewspaces / windows)
+    setAcceptDrops(true);
+
+    m_layout.tabBarLayout = hLayout;
+    m_layout.mainLayout = layout;
+
+    // init tab bar
     tabBarToggled();
 }
 
+KateViewSpace::~KateViewSpace() = default;
+
 bool KateViewSpace::eventFilter(QObject *obj, QEvent *event)
 {
     QToolButton *button = qobject_cast<QToolButton *>(obj);
@@ -168,23 +203,58 @@ bool KateViewSpace::eventFilter(QObject *obj, QEvent *event)
     return false;
 }
 
-void KateViewSpace::statusBarToggled()
-{
-    KateUpdateDisabler updatesDisabled(m_viewManager->mainWindow());
-    for (const auto &[_, view] : m_docToView) {
-        Q_UNUSED(_)
-        view->setStatusBarEnabled(m_viewManager->mainWindow()->showStatusBar());
-    }
-}
-
 void KateViewSpace::tabBarToggled()
 {
     KateUpdateDisabler updatesDisabled(m_viewManager->mainWindow());
-    m_historyBack->setVisible(m_viewManager->mainWindow()->showTabBar());
-    m_historyForward->setVisible(m_viewManager->mainWindow()->showTabBar());
-    m_tabBar->setVisible(m_viewManager->mainWindow()->showTabBar());
-    m_split->setVisible(m_viewManager->mainWindow()->showTabBar());
-    m_quickOpen->setVisible(m_viewManager->mainWindow()->showTabBar());
+
+    const bool show = m_viewManager->mainWindow()->showTabBar();
+    const bool urlBarVisible = m_viewManager->showUrlNavBar();
+
+    bool showButtons = true;
+
+    m_tabBar->setVisible(show);
+    if (show) {
+        m_layout.tabBarLayout->removeWidget(m_urlBar);
+        int afterTabLayout = m_layout.mainLayout->indexOf(m_layout.tabBarLayout);
+        m_layout.mainLayout->insertWidget(afterTabLayout + 1, m_urlBar);
+        showButtons = true;
+    } else if (!show && !urlBarVisible) {
+        // both hidden, hide buttons as well
+        showButtons = false;
+    } else if (!show && urlBarVisible) {
+        // UrlBar is still visible. Move it up to take the place of tabbar
+        int insertAt = m_layout.tabBarLayout->indexOf(m_historyForward) + 1;
+        m_layout.tabBarLayout->insertWidget(insertAt, m_urlBar);
+        showButtons = true;
+    }
+
+    m_historyBack->setVisible(showButtons);
+    m_historyForward->setVisible(showButtons);
+    m_split->setVisible(showButtons);
+    m_quickOpen->setVisible(showButtons);
+}
+
+void KateViewSpace::urlBarToggled(bool show)
+{
+    const bool tabBarVisible = m_viewManager->mainWindow()->showTabBar();
+    bool showButtons = true;
+    if (!show && !tabBarVisible) {
+        // Tab bar was already hidden, now url bar is also hidden
+        // so hide the buttons as well
+        showButtons = false;
+    } else if (show && !tabBarVisible) {
+        // Tabbar is hidden, but we now need to show url bar
+        // Move it up to take the place of tabbar
+        int insertAt = m_layout.tabBarLayout->indexOf(m_historyForward) + 1;
+        m_layout.tabBarLayout->insertWidget(insertAt, m_urlBar);
+        // make sure buttons are visible
+        showButtons = true;
+    }
+
+    m_historyBack->setVisible(showButtons);
+    m_historyForward->setVisible(showButtons);
+    m_split->setVisible(showButtons);
+    m_quickOpen->setVisible(showButtons);
 }
 
 KTextEditor::View *KateViewSpace::createView(KTextEditor::Document *doc)
@@ -197,8 +267,8 @@ KTextEditor::View *KateViewSpace::createView(KTextEditor::Document *doc)
      */
     KTextEditor::View *v = doc->createView(stack, m_viewManager->mainWindow()->wrapper());
 
-    // set status bar to right state
-    v->setStatusBarEnabled(m_viewManager->mainWindow()->showStatusBar());
+    // our framework relies on the existence of the status bar
+    v->setStatusBarEnabled(true);
 
     // restore the config of this view if possible
     if (!m_group.isEmpty()) {
@@ -243,6 +313,16 @@ void KateViewSpace::removeView(KTextEditor::View *v)
     // ...and now: remove from view space
     stack->removeWidget(v);
 
+    // Remove the doc now!
+    // Why do this now? Because otherwise it messes up the LRU
+    // because we get two "currentChanged" signals
+    // - First signal when we "showView" below
+    // - Second comes soon after when v->document() is destroyed
+    // Handling (blocking) both signals here is necessary
+    m_tabBar->blockSignals(true);
+    documentDestroyed(v->document());
+    m_tabBar->blockSignals(false);
+
     // switch to most recently used rather than letting stack choose one
     // (last element could well be v->document() being removed here)
     for (auto rit = m_registeredDocuments.rbegin(); rit != m_registeredDocuments.rend(); ++rit) {
@@ -330,7 +410,7 @@ KTextEditor::View *KateViewSpace::currentView()
     return qobject_cast<KTextEditor::View *>(stack->currentWidget());
 }
 
-bool KateViewSpace::isActiveSpace()
+bool KateViewSpace::isActiveSpace() const
 {
     return m_isActiveSpace;
 }
@@ -395,6 +475,125 @@ void KateViewSpace::registerDocument(KTextEditor::Document *doc)
     connect(m_tabBar, &KateTabBar::currentChanged, this, &KateViewSpace::changeView);
 }
 
+void KateViewSpace::closeDocument(KTextEditor::Document *doc)
+{
+    // If this is the only view of the document,
+    // OR the doc has no views yet
+    // just close the document and it will take
+    // care of removing the view + cleaning up the doc
+    if (doc->views().size() <= 1) {
+        m_viewManager->slotDocumentClose(doc);
+    } else {
+        auto it = m_docToView.find(doc);
+        Q_ASSERT(it != m_docToView.end());
+        // We have other views of this doc in other viewspaces
+        // Just remove the view
+        m_viewManager->deleteView(it->second);
+    }
+
+    /**
+     * if this was the last doc, let viewManager know we are empty
+     */
+    if (m_registeredDocuments.isEmpty() && m_tabBar->count() == 0) {
+        Q_EMIT viewSpaceEmptied(this);
+    }
+}
+
+bool KateViewSpace::acceptsDroppedTab(const QMimeData *md)
+{
+    if (auto tabMimeData = qobject_cast<const TabMimeData *>(md)) {
+        return this != tabMimeData->sourceVS && // must not be same viewspace
+            viewManger() == tabMimeData->sourceVS->viewManger() && // for now we don't support dropping into different windows
+            !hasDocument(tabMimeData->doc);
+    }
+    return TabMimeData::hasValidData(md);
+}
+
+void KateViewSpace::dragEnterEvent(QDragEnterEvent *e)
+{
+    if (acceptsDroppedTab(e->mimeData())) {
+        m_dropIndicator.reset(new QRubberBand(QRubberBand::Rectangle, this));
+        m_dropIndicator->setGeometry(rect());
+        m_dropIndicator->show();
+        e->acceptProposedAction();
+        return;
+    }
+
+    QWidget::dragEnterEvent(e);
+}
+
+void KateViewSpace::dragLeaveEvent(QDragLeaveEvent *e)
+{
+    m_dropIndicator.reset();
+    QWidget::dragLeaveEvent(e);
+}
+
+void KateViewSpace::dropEvent(QDropEvent *e)
+{
+    if (auto mimeData = qobject_cast<const TabMimeData *>(e->mimeData())) {
+        m_viewManager->moveViewToViewSpace(this, mimeData->sourceVS, mimeData->doc);
+        m_dropIndicator.reset();
+        e->accept();
+        return;
+    }
+    auto droppedData = TabMimeData::data(e->mimeData());
+    if (droppedData.has_value()) {
+        auto doc = KateApp::self()->documentManager()->openUrl(droppedData.value().url);
+        auto view = m_viewManager->activateView(doc);
+        if (view) {
+            view->setCursorPosition({droppedData.value().line, droppedData.value().col});
+            m_dropIndicator.reset();
+            e->accept();
+            return;
+        }
+    }
+
+    QWidget::dropEvent(e);
+}
+
+bool KateViewSpace::hasDocument(KTextEditor::Document *doc) const
+{
+    return m_registeredDocuments.contains(doc) && (m_docToView.find(doc) != m_docToView.end());
+}
+
+KTextEditor::View *KateViewSpace::takeView(KTextEditor::Document *doc)
+{
+    auto it = m_docToView.find(doc);
+    if (it == m_docToView.end()) {
+        return nullptr;
+    }
+    auto *view = it->second;
+    // remove it from the stack
+    stack->removeWidget(view);
+    // remove it from our doc->view mapping
+    m_docToView.erase(it);
+    documentDestroyed(doc);
+
+    // Did we just loose our last doc?
+    // Send a delayed signal. Delay is important as we want to kill
+    // the viewspace after the view transfer was done
+    if (m_tabBar->count() == 0 && m_registeredDocuments.empty()) {
+        QMetaObject::invokeMethod(
+            this,
+            [this] {
+                Q_EMIT viewSpaceEmptied(this);
+            },
+            Qt::QueuedConnection);
+    }
+
+    return view;
+}
+
+void KateViewSpace::addView(KTextEditor::View *v)
+{
+    registerDocument(v->document());
+    m_docToView[v->document()] = v;
+    // We must not already have this widget
+    Q_ASSERT(stack->indexOf(v) == -1);
+    stack->addWidget(v);
+    showView(v);
+}
+
 void KateViewSpace::documentDestroyed(QObject *doc)
 {
     /**
@@ -442,15 +641,10 @@ void KateViewSpace::updateDocumentUrl(KTextEditor::Document *doc)
 
 void KateViewSpace::updateDocumentState(KTextEditor::Document *doc)
 {
-    QIcon icon;
-    if (doc->isModified()) {
-        icon = QIcon::fromTheme(QStringLiteral("document-save"));
-    }
-
     // update tab button if available, might not be the case for tab limit set!
     const int buttonId = m_tabBar->documentIdx(doc);
     if (buttonId >= 0) {
-        m_tabBar->setTabIcon(buttonId, icon);
+        m_tabBar->update(m_tabBar->tabRect(buttonId));
     }
 }
 
@@ -473,7 +667,7 @@ void KateViewSpace::closeTabRequest(int idx)
         return;
     }
 
-    m_viewManager->slotDocumentClose(doc);
+    closeDocument(doc);
 }
 
 void KateViewSpace::createNewDocument()
@@ -540,6 +734,13 @@ void KateViewSpace::closeTabWithWidget(QWidget *widget)
     }
 }
 
+void KateViewSpace::focusNavigationBar()
+{
+    if (!m_urlBar->isHidden()) {
+        m_urlBar->open();
+    }
+}
+
 void KateViewSpace::addPositionToHistory(const QUrl &url, KTextEditor::Cursor c, bool calledExternally)
 {
     // We don't care about invalid urls (Fixed Diff View / Untitled docs)
@@ -815,8 +1016,12 @@ void KateViewSpace::restoreConfig(KateViewManager *viewMan, const KConfigBase *c
         auto *doc = KateApp::self()->documentManager()->documentList().first();
         if (!fn.isEmpty()) {
             QUrl url(fn);
-            KateApp::self()->documentManager()->documentInfo(doc)->doPostLoadOperations =
-                !url.isLocalFile() && (KateApp::self()->hasCursorInArgs() || url.hasQuery());
+            auto *docInfo = KateApp::self()->documentManager()->documentInfo(doc);
+            if (KateApp::self()->hasCursorInArgs()) {
+                docInfo->startCursor = KateApp::self()->cursorFromArgs();
+            } else if (url.hasQuery()) {
+                docInfo->startCursor = KateApp::self()->cursorFromQueryString(url);
+            }
         }
         viewMan->createView(doc, this);
     }
